#========================================================================================
# package      DE_EPAGES::Order::t::CartCreationResource
# base         DE_EPAGES::Shop::API::TestImportXML
# state        public
#----------------------------------------------------------------------------------------
# description  Integration test for CartCreationResource.
#========================================================================================
package DE_EPAGES::Order::t::CartCreationResource;
use base DE_EPAGES::Shop::API::TestImportXML;

use DE_EPAGES::Order::API::Resource::CartCreationResource;
use Test::More;

use strict;



sub shopAlias {
    return 'RestApiTestShop';
}



sub importXMLPath {
    return 'ShopXMLs/Shop_for_CartCreationResource_test.xml';
}



sub test {
    my $self = shift;
    $self->test_postCart();
    done_testing();
}



sub test_postCart {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $hRequestData = $self->_createPostCartRequest();
    my $Resource = DE_EPAGES::Order::API::Resource::CartCreationResource->new();


    my $hBasketData = $Resource->post($Shop, $hRequestData);


    my $LineItemContainer = $hBasketData->{'lineItemContainer'};
    my $aProductLineItems = $LineItemContainer->{'productLineItems'};
    my $ProductLineItem = $aProductLineItems->[0];

    is($hBasketData->{'minCartValue'}, undef, "$SubTestName - test if minimum cart value is undefined");
    is($hBasketData->{'shippingAddress'}, undef, "$SubTestName - test if shipping address is undefined");
    is($hBasketData->{'billingAddress'}, undef, "$SubTestName - test if billing address is undefined");

    is_deeply($LineItemContainer->{'grandTotal'}, $self->_createdExpected_grandTotal(), "$SubTestName - test if grandTotal in line item container is as expected.");
    is_deeply($LineItemContainer->{'totalBeforeTax'}, $self->_createdExpected_totalBeforeTax(), "$SubTestName - test if grandTotal in line item container is as expected.");
    is_deeply($LineItemContainer->{'totalTax'}, $self->_createdExpected_totalTax(), "$SubTestName - test if totalTax in line item container is as expected.");
    is_deeply($LineItemContainer->{'lineItemsSubTotal'}, $self->_createdExpected_lineItemsSubTotal(), "$SubTestName - test if lineItemsSubTotal in line item container is as expected.");

    is_deeply($ProductLineItem->{'images'}, [], "$SubTestName - test if product line item does not contain images.");
    is($ProductLineItem->{'sku'}, 'TestProduct', "$SubTestName - test if sku is as expected");
    is($ProductLineItem->{'name'}, 'TestProduct', "$SubTestName - test if name is as expected");
    is_deeply($ProductLineItem->{'quantity'}, $self->_createdExpected_quantity(), "$SubTestName - test if quantity in line item container is as expected.");
    is_deeply($ProductLineItem->{'lineItemPrice'}, $self->_createdExpected_lineItemPrice(), "$SubTestName - test if lineItemPrice in line item container is as expected.");
    is_deeply($ProductLineItem->{'singleItemPrice'}, $self->_createdExpected_singleItemPrice(), "$SubTestName - test if singleItemPrice in line item container is as expected.");

    return;
}



sub _createPostCartRequest() {
    my $self = shift;
    return {
        'currency' => 'EUR',
        'taxType' => 'GROSS',
        'locale' => 'de_DE',
        'lineItems' => [
            {
                'productId' => '53AAD424-2BA4-9660-8182-AC1432AA1F2B',
                'quantity' => '1',
            }
        ],
    };
}



sub _createdExpected_grandTotal {
    my $self = shift;
    return {
        'currency' => 'EUR',
        'taxType' => 'GROSS',
        'amount' => '74.14',
        'formatted' => "74,14 \x{20ac}"
    };
}



sub _createdExpected_totalBeforeTax {
    my $self = shift;
    return {
        'currency' => 'EUR',
        'taxType' => 'NET',
        'amount' => '62.3',
        'formatted' => "62,30 \x{20ac}"
    };
}



sub _createdExpected_totalTax {
    my $self = shift;
    return {
        'currency' => 'EUR',
        'taxType' => 'NONE',
        'amount' => '11.84',
        'formatted' => "11,84 \x{20ac}"
    };
}



sub _createdExpected_lineItemsSubTotal {
    my $self = shift;
    return {
        'currency' => 'EUR',
        'taxType' => 'GROSS',
        'amount' => '72.95',
        'formatted' => "72,95 \x{20ac}"
    };
}



sub _createdExpected_lineItemPrice {
    my $self = shift;
    return {
        'currency' => 'EUR',
        'taxType' => 'GROSS',
        'amount' => '72.95',
        'formatted' => "72,95 \x{20ac}"
    };
}



sub _createdExpected_singleItemPrice {
    my $self = shift;
    return {
        'currency' => 'EUR',
        'taxType' => 'GROSS',
        'amount' => '72.95',
        'formatted' => "72,95 \x{20ac}"
    };
}



sub _createdExpected_quantity {
    my $self = shift;
    return {
        'unit' => "St\x{fc}ck",
        'amount' => 1
    };
}

__PACKAGE__->runScript();

1;