package DE_EPAGES::Order::t::CartResource;
use base DE_EPAGES::Shop::API::Test;

use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Order::API::Resource::Cart;
use URI::QueryParam;
use URI::Escape;
use URI;
use Test::More;
use Test::Exception;
use Data::Dumper;

use strict;



## no critic qw( ProhibitMagicNumbers )

sub test {
    my $self = shift;

    $self->test_GetByID('cartTester01','cartTest01', 'en_GB', {
        minCartValue => undef,
        billingAddress => undef,
        shippingAddress => undef,
        lineItemContainer => {
            grandTotal => {
                currency => 'GBP',
                taxType => 'GROSS',
                amount => '79.95',
                formatted => "79.95",
            },
            totalBeforeTax => {
                currency => 'GBP',
                taxType => 'NET',
                amount => '67.18',
                formatted => "67.18",
            },
            totalTax => {
                currency => 'GBP',
                taxType => 'NONE',
                amount => '12.77',
                formatted => "12.77",
            },
            lineItemsSubTotal => {
                currency => 'GBP',
                taxType => 'GROSS',
                amount => '72.95',
                formatted => "72.95",
            },
            shippingPrice => {
                currency => 'GBP',
                taxType => 'GROSS',
                amount => '7',
                formatted => "7.00",
            },
            productLineItems => [
                {
                    lineItemId => '53AD3774-B3DC-155B-3E97-AC1432AA8AC2',
                    productId => '53AAD424-2BA4-9660-8182-AC1432AA1F2B',
                    sku => 'lt_0401107001',
                    name => 'Leatherman Tool Survival',
                    quantity => {
                        unit => 'piece(s)',
                        amount => '1'
                    },
                    singleItemPrice => {
                        currency => 'GBP',
                        taxType => 'GROSS',
                        amount => '72.95',
                        formatted => "72.95",
                    },
                    lineItemPrice => {
                        currency => 'GBP',
                        taxType => 'GROSS',
                        amount => '72.95',
                        formatted => "72.95",
                    },
                    images => [],
                    links => [
                        {
                            rel => 'product',
                            href => 'https://test-domain/api/products/53AAD424-2BA4-9660-8182-AC1432AA1F2B'
                        },
                    ],
                },
            ],
        },
    }, 'https://test-domain/epages/Store.sf/?ObjectPath=/Test/TestShop/AnonymousUsers/cartTester01/Baskets/cartTest01&ChangeAction=PickupBasket');

    $self->test_GetByID('cartTester02','cartTest02', 'de_DE', {
        minCartValue => undef,
        billingAddress => {
            country => 'DE',
            zipCode => '08157',
            firstName => 'Gaby',
            salutation => undef,
            vatId => undef,
            street => 'Musterweg 42',
            state => undef,
            city => 'Musterhausen',
            title => undef,
            lastName => 'Mustermann',
            streetDetails => undef,
            birthday => '1970-08-18',
            company => undef,
            emailAddress => 'hund@epages.de'
        },
        shippingAddress => {
            country => 'DE',
            zipCode => '08157',
            firstName => 'Fred',
            salutation => undef,
            vatId => undef,
            street => 'Musterweg 42',
            state => undef,
            city => 'Musterhausen',
            title => undef,
            lastName => 'Mustermann',
            streetDetails => undef,
            birthday => undef,
            company => undef,
            emailAddress => undef
        },
        lineItemContainer => {
            grandTotal => {
                currency => 'EUR',
                taxType => 'GROSS',
                amount => '34.35',
                formatted => "34,35 \x{20ac}",
            },
            totalBeforeTax => {
                currency => 'EUR',
                taxType => 'NET',
                amount => '28.87',
                formatted => "28,87 \x{20ac}",
            },
            totalTax => {
                currency => 'EUR',
                taxType => 'NONE',
                amount => '5.48',
                formatted => "5,48 \x{20ac}",
            },
            lineItemsSubTotal => {
                currency => 'EUR',
                taxType => 'GROSS',
                amount => '26.9',
                formatted => "26,90 \x{20ac}",
            },
            shippingPrice => {
                currency => 'EUR',
                taxType => 'GROSS',
                amount => '7.45',
                formatted => "7,45 \x{20ac}",
            },
            productLineItems => [
                {
                    lineItemId => '53AD3774-8BE0-3028-D733-AC1432AA8A71',
                    productId => '53AA9617-0C07-5434-801A-AC1432AA7911',
                    sku => 'cg_0101004270',
                    name => 'Campingaz CV270 Ventilgaskartusche',
                    quantity => {
                        unit => "St\x{fc}ck",
                        amount => '1'
                    },
                    singleItemPrice => {
                        currency => 'EUR',
                        taxType => 'GROSS',
                        amount => '3.95',
                        formatted => "3,95 \x{20ac}",
                    },
                    lineItemPrice => {
                        currency => 'EUR',
                        taxType => 'GROSS',
                        amount => '3.95',
                        formatted => "3,95 \x{20ac}",
                    },
                    images => [],
                    links => [
                        {
                            rel => 'product',
                            href => 'https://test-domain/api/products/53AA9617-0C07-5434-801A-AC1432AA7911'
                        },
                    ],
                },
                {
                    lineItemId => '53AD3774-0D65-3199-0752-AC1432AA8AAC',
                    productId => '53AA962B-1F09-B778-E3AF-AC1432AA7986',
                    sku => 'cg_0100504001',
                    name => 'Campingaz Twister 270',
                    quantity => {
                        unit => "St\x{fc}ck",
                        amount => '1'
                    },
                    singleItemPrice => {
                        currency => 'EUR',
                        taxType => 'GROSS',
                        amount => '22.95',
                        formatted => "22,95 \x{20ac}",
                    },
                    lineItemPrice => {
                        currency => 'EUR',
                        taxType => 'GROSS',
                        amount => '22.95',
                        formatted => "22,95 \x{20ac}",
                    },
                    images => [],
                    links => [
                        {
                            rel => 'product',
                            href => 'https://test-domain/api/products/53AA962B-1F09-B778-E3AF-AC1432AA7986'
                        },
                    ],
                },
            ],
        }
    }, 'https://test-domain/epages/Store.sf/?ObjectPath=/Test/TestShop/AnonymousUsers/cartTester02/Baskets/cartTest02&ChangeAction=PickupBasket');

    #place her later on the delete and put test
    done_testing();
}

sub test_GetByID {
    my $self = shift;
    my ( $UserAlias, $BasketAlias, $LocaleID, $hReference, $ExpectedUrlWithoutPickupToken ) = @_;
    note "test_GetByID $UserAlias, $BasketAlias, $LocaleID";

    my $Cart = $self->shop->folder('AnonymousUsers')->child($UserAlias)->folder('Baskets')->child($BasketAlias);
    $hReference->{cartId} = $Cart->guid;

    my $Resource = DE_EPAGES::Order::API::Resource::Cart->new;

    my $Request1 = HTTP::Request->new(GET => 'http://www.example.com/api/carts/' . $Cart->guid . '/');
    ok($Resource->match($Request1), 'url1 matches');

    my $Request2 = HTTP::Request->new(GET => 'http://www.example.com/api/carts/' . $Cart->guid);
    ok($Resource->match($Request2), 'url2 matches');

    my $hData = $Resource->get( $Cart, $LocaleID );
    ok $hData, "get cart";

    my $CheckoutUrl = delete $hData->{'checkoutUrl'};
    my $Uri = URI->new($CheckoutUrl);
    $Uri->query_param_delete('PickupToken');
    my $UrlWithoutPickupToken = uri_unescape($Uri->as_string());
    is( $UrlWithoutPickupToken, $ExpectedUrlWithoutPickupToken, 'test if checkout url has right shape.');

    is_deeply( $hData, $hReference, 'cart result deeply checked') or diag Dumper($hData);
}

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';
    $self->shop->set({
        DomainName => 'test-domain',
        HasSSLCertificate => 1
    });

    note 'importing cartTest.xml';
    ImportObjects('cartTest.xml', $self->shop->pathString);
}

__PACKAGE__->runScript;
