#========================================================================================
# package      DE_EPAGES::Order::t::unit::BasketAddress
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::BasketAddress
#========================================================================================
package DE_EPAGES::Order::t::unit::BasketAddress;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Order::API::BasketAddress;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw (GetParametersFromMockifyCall);
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_setBillingAddress();
    $self->test_removeBillingAddress();

    $self->test_setShippingAddress();
    $self->test_removeShippingAddress();

    done_testing();
}



sub test_setBillingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $BasketAddress = $self->_createBasketAddress($Basket);
    my $hAddressData = $self->_createAddressRequestData();

    $BasketAddress->setAddress($hAddressData, 'Billing');

    my $aGetPKeyCountryByCode2CallParams = GetParametersFromMockifyCall($BasketAddress, '_GetPKeyCountryByCode2');
    is($aGetPKeyCountryByCode2CallParams->[0], 'myCountry', "$SubTestName - test if GetPKeyCountryByCode2 was called with correct parameter.");

    my $aCheckBasketAddressCallParams = GetParametersFromMockifyCall($BasketAddress, '_CheckBasketAddress');
    is($aCheckBasketAddressCallParams->[2], 'BillingAddress', "$SubTestName - test if CheckBasketAddress was called with parameter BillingAddress.");
    is($aCheckBasketAddressCallParams->[3], '0', "$SubTestName - test if CheckBasketAddress was called with UsesShipping-parameter 0.");

    my $AddressFolder = $self->{'__objectsToCheck'}->{'AddressFolder'};
    my $aInsertChildCallParams = GetParametersFromMockifyCall($AddressFolder, 'insertChild');
    is($aInsertChildCallParams->[0], 'Address', "$SubTestName - test if first parameter of insertChild call is Address.");
    is_deeply($aInsertChildCallParams->[1], $self->_createdExpectedAddressData(), "$SubTestName - test if first parameter of insertChild call is Address.");
}



sub test_removeBillingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasketForAddressDeletion();
    my $BasketAddress = $self->_createBasketAddress($Basket);

    $BasketAddress->removeAddress('Billing');

    my $BasketUser = $Basket->get('User');
    is($Basket->{'BillingAddress'}, undef, "$SubTestName - test if BillingAddress was removed from Basket");
    is($BasketUser->{'BillingAddress'}, undef, "$SubTestName - test if BillingAddress was removed from BasketUser");
}



sub test_setShippingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $BasketAddress = $self->_createBasketAddress($Basket);
    my $hAddressData = $self->_createAddressRequestData();

    $BasketAddress->setAddress($hAddressData, 'Shipping');

    my $aGetPKeyCountryByCode2CallParams = GetParametersFromMockifyCall($BasketAddress, '_GetPKeyCountryByCode2');
    is($aGetPKeyCountryByCode2CallParams->[0], 'myCountry', "$SubTestName - test if GetPKeyCountryByCode2 was called with correct parameter.");

    my $aCheckBasketAddressCallParams = GetParametersFromMockifyCall($BasketAddress, '_CheckBasketAddress');
    is($aCheckBasketAddressCallParams->[2], 'ShippingAddress', "$SubTestName - test if CheckBasketAddress was called with parameter ShippingAddress.");
    is($aCheckBasketAddressCallParams->[3], '1', "$SubTestName - test if CheckBasketAddress was called with UsesShipping-parameter 0.");

    my $AddressFolder = $self->{'__objectsToCheck'}->{'AddressFolder'};
    my $aInsertChildCallParams = GetParametersFromMockifyCall($AddressFolder, 'insertChild');
    is($aInsertChildCallParams->[0], 'Address', "$SubTestName - test if first parameter of insertChild call is Address.");
    is_deeply($aInsertChildCallParams->[1], $self->_createdExpectedAddressData(), "$SubTestName - test if first parameter of insertChild call is Address.");
}



sub test_removeShippingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasketForAddressDeletion();
    my $BasketAddress = $self->_createBasketAddress($Basket);

    $BasketAddress->removeAddress('Shipping');

    my $BasketUser = $Basket->get('User');
    is($Basket->{'ShippingAddress'}, undef, "$SubTestName - test if ShippingAddress was removed from Basket");
    is($BasketUser->{'ShippingAddress'}, undef, "$SubTestName - test if ShippingAddress was removed from BasketUser");
}



### helper methods ###



sub _createBasketAddress {
    my $self = shift;
    my ($Basket) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Order::API::BasketAddress',
        [$Basket]
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_GetPKeyCountryByCode2',
        'DE',
        ['string']
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_CheckBasketAddress',
        1,
        ['object', 'object', 'string', 'int']
    );
    $Mockify->addMockWithReturnValue(
        '_LoadRootObject',
        $self->_createSystemObject()
    );
    $Mockify->addMockWithReturnValue(
        '_createSOAPDateTimeFormatXML',
        $self->_createSOAPDateTimeFormatXML()
    );
    $Mockify->addMock('_Transaction', sub {
        my $MockedSelf = shift;
        my ($cCode, $cRollback) = @_;
        $cCode->();
        return;
    });
    return $Mockify->getMockObject();
}



sub _createBasket {
    my $self = shift;
    my $hAttributes = {
        'User' => $self->_createBasketUser(),
        'ShippingAddress' => undef,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', $hAttributes);
}



sub _createBasketForAddressDeletion {
    my $self = shift;
    my $hAttributes = {
        'User' => $self->_createBasketUserForAddressDeletion(),
        'BillingAddress' => $self->_createAddress(),
        'ShippingAddress' => $self->_createAddress(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', $hAttributes);
}



sub _createBasketUser {
    my $self = shift;
    my $hAttributes = {
        'LocaleID' => 'de_DE',
    };
    my $BasketUser = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Permission::API::Object::User', $hAttributes);
    $BasketUser->mock('folder', sub {
        return $self->_createAddressesFolder();
    });
    return $BasketUser;
}



sub _createBasketUserForAddressDeletion {
    my $self = shift;
    my $hAttributes = {
        'BillingAddress' => $self->_createAddress(),
        'ShippingAddress' => $self->_createAddress(),
    };
    my $BasketUser = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Permission::API::Object::User', $hAttributes);
    return $BasketUser;
}



sub _createAddressesFolder {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Object::API::Object::Object',
        []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('nextAlias', 'TestAlias', ['string']);
    $Mockify->addMockWithReturnValueAndParameterCheck('insertChild', undef, ['string', 'hashref']);
    my $AddressFolder = $Mockify->getMockObject();
    $self->{'__objectsToCheck'}->{'AddressFolder'} = $AddressFolder;
    return $AddressFolder;
}



sub _createAddress {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Address::API::Object::Address',
        []
    );
    $Mockify->addMockWithReturnValue('delete', undef);
    return $Mockify->getMockObject();
}



sub _createSOAPDateTimeFormatXML {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::XML::API::SOAPDateTimeFormatXML',
        []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('parse_datetime', '01.02.1903', ['string', 'string']);
    return $Mockify->getMockObject();
}



sub _createSystemObject {
    my $self = shift;
    my $hAttributes = {
        'TimeZone' => 'Europe/Berlin '
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site', $hAttributes);
}



sub _createAddressRequestData {
    return {
        'birthday' => '1903-02-01',
        'city' => 'Testburg',
        'company' => 'epages',
        'country' => 'myCountry',
        'emailAddress' => 'email@test.org',
        'firstName' => 'Joe',
        'lastName' => 'Doe',
        'salutation' => 'Salve!',
        'state' => 'HH',
        'street' => 'Teststr.',
        'streetDetails' => '2nd Backyard',
        'title' => 'Dr.',
        'vatId' => '123-456-7-890',
        'zipCode' => '12345',
    };
}



sub _createdExpectedAddressData {
    my $self = shift;
    return {
      'Alias' => 'TestAlias',
      'Birthday' => '01.02.1903',
      'City' => 'Testburg',
      'Company' => 'epages',
      'CountryID' => 'DE',
      'EMail' => 'email@test.org',
      'FirstName' => 'Joe',
      'LastName' => 'Doe',
      'Salutation' => 'Salve!',
      'State' => 'HH',
      'Street' => 'Teststr.',
      'Street2' => '2nd Backyard',
      'Title' => 'Dr.',
      'VATID' => '123-456-7-890',
      'Zipcode' => 12345,
    };
}

__PACKAGE__->runScript();